@@ -404,6 +404,14 @@ module NumRu
 	          "averaged  "+@name+":#{mn}..#{mx}" ]
       end
 
+      def draw_positive
+	# Returns the direction to plot the axis
+	# * true: axis should be drawn in the increasing order(to right/upward)
+	# * false: axis should be drawn in the decreasing order
+	# * nil: not specified (VArray's default if not overridden in 
+        #   subclasses)
+	@pos.axis_draw_positive
+      end
 
       #########  private methods ####################################
 
