@@ -71,8 +71,6 @@ Test script path is 'test/test_ep_flux.rb' in expand dir of gphys.tar.gz.
     * ((<preparate_for_vector_on_merdional_section>))
       * preparate for ((<GGraph::vector_on_merdional_section>)) 
         in vector_on_merdional_section.rb
-    * ((<preparate_for_strm_rmean>))
-      * preparate for ((<strm_rmean>)) 
   * ((<Constants:>))
     * ((<Deriv_methods>))
       * derivative method names.
@@ -410,35 +408,10 @@ Module functions of EP_Flux Operater for GPhys.
     * va_z(VArray): a VArray which represents z-coordinate.
     * was_proportional_to_p(True or False): flag original axis proportional to 
                                             pressure or z.
----preparate_for_divergence_on_merdional_section(xax, yax)
-
-   preparate for ((<divergence_on_merdional_section>)). 
-
-     (1) check ((<yax>)) if it is proportional to p 
-     (2) get axis ( a*sin(phi), z , cos(phi)) 
-
-    ARGUMENTS
-    * xax(VArray): a VArray which represents x axis.
-    * yax(VArray): a VArray which represents y axis.
-
-    RETURN VALUE
-    * va_a_sin_phi(VArray): a VArray which represents x-coordinate(radius * sin(phi)).
-    * va_z(VArray): a VArray which represents z-coordinate.
-    * va_cos_phi(VArray): a VArray which represents x-coordinate(radius * cos(phi)).
-    * was_proportional_to_p(True or False): flag original axis proportional to 
-                                            pressure or z.
 
 ---strm_rmean(v_rmean, yzdims=[0,1])
 
-    !! CAUTION !! 
-    Now this method might show good precision, but we have not enough test 
-    and debuged yet. So, there is no gurantee. please use with your 
-    responsibility.
-    
     Calculate mass stream function for residual zonal mean circulation. 
-    Residual zonal mean circulation defined of this module is nondivergent if
-    products a value like a reference density profile (sigma), thus one can 
-    define mass stream function Psi. 
 
     ARGUMENTS
     * v_rmean (GPhys): a GPhys which is residual zonal mean V.
@@ -447,22 +420,6 @@ Module functions of EP_Flux Operater for GPhys.
     RETURN VALUE
     * gp_strm (GPhys): a GPhys which is mass stream function on merdional section.
 
----preparate_for_strm_rmean(gp_data, gp_p, z_dim)
-
-   preparate for ((<strm_rmean>)). its works
-
-     (1) expand 2 grid in the top boundary of pressure coordinate.
-     (2) interpolation with linear extention.
-     (3) top boundary condition is "p = 0".
-
-    ARGUMENTS
-    * gp_data (GPhys): a GPhys which you wants to expand to.
-    * gp_p    (GPhys): a GPhys which represents axis you wants expand to.
-    * z_dim   (Numric): a Numeric which represents dimension of pressure.
-
-    RETURN VALUE
-    * ext_gp_data (GPhys): a GPhys which extend top boundary.
-
 ==Constants:
 
 ---Deriv_methods
@@ -492,10 +449,8 @@ module NumRu
                                     # gravitational acceleration in the surface
       @@p00          = UNumeric.new(1.0E5,"Pa") # reference surface pressure
       @@cp           = UNumeric.new(1004.0, "J.K-1.kg-1")
-#      @@cp = 1004.0
                                            # specific heat at constant pressure
       @@gas_const    = UNumeric.new(287.0, "J.K-1.kg-1")
-#      @@gas_const = 287.0
                                            # gas constant per molecular mass
       @@deriv_method = Proc.new{|*args|
         GPhys::Derivative::threepoint_O2nd_deriv(*args)
@@ -852,111 +807,54 @@ module NumRu
 	return gp_aphi.data, gp_z.data, was_proportional_to_p
       end
 
-      def preparate_for_divergence_on_merdional_section(xax, zax)
-	gp_x, gp_z =   make_gphys(xax, zax)    # make gphys from axis	
-	gp_phi =       to_rad_if_deg(gp_x)     # deg => rad (unit convesion)
-	gp_a_sin_phi = @@radius * sin(gp_phi)  # radius * phi
-	gp_cos_phi =   cos(gp_phi)  # radius * phi
-	# check zax units, if proportional to z or p
-	if ( gp_z.data.units =~ Units.new('Pa') )
-	  was_proportional_to_p = true 
-	elsif ( gp_z.data.units =~ Units.new('m') )
-	  was_proportional_to_p = false
-	else
-	  raise ArgumentError,'unit of zax     #{gp_z.data.units} must be 
-                               compatible to length or pressure.'
-	end
-        gp_z = to_z_if_pressure(gp_z)   # convert to z if gp_z is pressure
-	gp_z[0] = +1E-6 if gp_z.data.val[0] == -0.0
-	return gp_a_sin_phi.data, gp_z.data, gp_cos_phi, was_proportional_to_p
-      end
-
-      def strm_rmean(v_rmean, yzdims=[0,1])
-	## !!CAUTION!!
-        ##  Now this method might show good performance, but we have not enough 
-        ##  test and debuged yet. So there is no gurantee. please use with 
-        ##  your responsibility.
+      def strm_rmean(gp_v, yzdims=[0,1])
 
 	raise ArgumentError,"yzdims's size (#{yzdims.size}) must be 2." if yzdims.size != 2
 	## get axis and name
 	lat_dim, z_dim = yzdims             # Index of dims
-	ax_lat = v_rmean.axis(lat_dim)      # Axis of latitude
-	ax_z   = v_rmean.axis(z_dim)        # Axis of vertical
+	ax_lat = gp_v.axis(lat_dim)      # Axis of latitude
+	ax_z   = gp_v.axis(z_dim)        # Axis of vertical
         lat_nm, z_nm = ax_lat.pos.name, ax_z.pos.name
 	gp_lat, gp_z = make_gphys(ax_lat, ax_z)
 	## convert
-	gp_p =   to_p_if_altitude(gp_z)     # z => p=p00exp(-z/H) (units-based)
-	gp_lat = to_rad_if_deg(gp_lat)      # deg => rad (unit convesion)
-	p_nm = gp_p.data.name               # only use in integration
+	gp_lat =   to_rad_if_deg(gp_lat)      # deg. to rad.
+	gp_p   =   to_p_if_altitude(gp_z)     # z => p=p00exp(-z/H) (units-based) and "Pa"
 
-	## replace grid (without duplicating data)
-	grid =    v_rmean.grid_copy
-	cp_grid = v_rmean.grid_copy         # saved to use in outputs
-	grid.axis(lat_nm).pos = gp_lat.data
-	grid.axis(z_nm).pos = gp_p.data
-	v_rmean = GPhys.new(grid, v_rmean.data)
+	## copy grid 
+	grid =    gp_v.grid_copy
 
 	## calculate stream function
-	na_v = v_rmean.data.val.dup         # for integration box 
-	pres = gp_p.data.val.dup
-	p_top = pres.min                    # minimal value in pressure coordinate
-	ext_v = preparate_for_strm_rmean(v_rmean, gp_p, z_dim)
-	idx = 0
-	## for devloping:: not use GPhys::integrate.
-	pres.each { |p_i|                   # integrate 
-	  na_v[*([true]*z_dim+[idx, false])] = \
-	  (ext_v.cut(p_nm=>0..p_i).integrate(p_nm)).data.val
-	  idx += 1
-	}
-	int_v = VArray.new( na_v, v_rmean.data, v_rmean.data.name )
-	gp_int_v   = GPhys.new(cp_grid, int_v)
+	na_v  = gp_v.data.val                  # for integration box 
+	int_v = gp_v.data.val.dup.fill!(0.0)   # for integration box 
+	pres  = gp_p.data.val.dup
+	if pres[0] < pres[-1]
+	  int_v[*([true]*z_dim+[0, false])] = 0.5*(na_v[*([true] + [0, false])])*pres[0]
+	  1.upto( pres.size-1 ) do |idx|
+	    dp = (pres[idx] - pres[idx-1])
+	    int_v[*([true]*z_dim+[idx, false])] = \
+  	      0.5 * (na_v[*([true] + [idx-1, false])] + na_v[*([true] + [idx, false])]) * dp \
+	      + int_v[*([true] + [idx-1, false])]
+	  end
+	else
+	  int_v[*([true]*z_dim+[-1, false])] = 0.5*(na_v[*([true] + [-1, false])])*pres[-1]
+	  ( pres.size-2 ).downto(0) do |idx|
+	    dp = (pres[idx] - pres[idx+1])
+	    int_v[*([true]*z_dim+[idx, false])] = \
+	      0.5 * (na_v[*([true] + [idx+1, false])] + na_v[*([true] + [idx, false])]) * dp \
+	      + int_v[*([true] + [idx+1, false])]
+	  end
+	end
+	int_v = VArray.new( int_v, gp_v.data, gp_v.data.name )
+	int_v.units = Units.new("Pa.m.s-1")
+	gp_int_v   = GPhys.new(grid, int_v)
         strm_rmean = gp_int_v * cos(gp_lat) * 2 * PI * @@radius / @@g_forces
 
 	## change attribute
 	strm_rmean.name = "strm_rmean"
-#	strm_rmean.units = "m3.s-1"
 	strm_rmean.set_att("long_name", "Residual mean mass stream function")
 
 	## convert with past grid
-	return GPhys.new(cp_grid, strm_rmean.data)
-      end
-
-      def preparate_for_strm_rmean(gp_data, gp_p, z_dim)
-	v_data = gp_data.data    ; v_p = gp_p.data
-	n_data = v_data.val      ; n_p = v_p.val
-	grid_data = gp_data.grid_copy
-	## expand top boundary
-	if n_p[0] < n_p[-1]
-	  ## top value
-	  p_top    = n_p[0]
-	  data_top = n_data[*([true]*z_dim + [0]  + [false])]
-	  ## preperate expand NArray
-	  ext_data = n_data[*([true]*z_dim   + [[0,0,0..(n_p.shape[0]-1)]]  + [false])]  
-	  ext_p =       n_p[*([[0,0,0..(n_p.shape[0]-1)]])]
-	  ## interpolation 
-	  ext_data[*([true]*z_dim + [0] + [false])] = 0
-	  ext_data[*([true]*z_dim + [1] + [false])] = data_top * 0.5
-	  ext_p[0]    = 0
-	  ext_p[1]    = p_top * 0.5
-	else
-	  ## top value
-	  p_top = n_p[-1]
-	  data_top = n_data[*([true]*z_dim + [-1]  + [false])]  
-	  ## preperate expand NArray
-	  ext_data = n_data[*([true]*z_dim   + [[0..(n_p.shape[0]-1),0,0]]  + [false])]  
-	  ext_p =       n_p[*([[0..(n_p.shape[0]-1),0,0]])]
-	  ## interpolation 
-	  ext_data[*([true]*z_dim + [-1] + [false])] = 0
-	  ext_data[*([true]*z_dim + [-2] + [false])] = data_top * 0.5
-	  ext_p[-1]    = 0
-	  ext_p[-2]    = p_top * 0.5
-	end
-	## make new VArray
-	v_data  = VArray.new( ext_data, v_data, v_data.name )
-	v_p  =    VArray.new( ext_p, v_p, v_p.name)
-	grid_data.axis(z_dim).pos = v_p
-	ext_gp_data = GPhys.new(grid_data, v_data)
-	return ext_gp_data
+	return strm_rmean
       end
       
     end  
