@@ -18,15 +18,7 @@ module NumRu
       module_function
 
       def each_along_dims_write(gphyses, files, loopdims, io_module)
-	case gphyses
-	when Array
-	  shape = gphyses[0].shape
-	  for i in 1...gphyses.length
-	    if shape != gphyses[i].shape
-	      raise "Shape of the GPhys objects must be the same"
-	    end
-	  end
-	else
+	if !gphyses.is_a?(Array)
 	  gphyses = [gphyses]     # put in an Array (if a single GPhys)
 	end
 	gp = gphyses[0]
@@ -72,6 +64,19 @@ module NumRu
 	  len *= gp.shape[i]
 	}
 
+	gphyses.each do |g|
+	  for i in 1...gphyses.length
+	    loopdimnames.each_with_index do |nm,i|
+	      if !g.axnames.include?( nm )
+		raise ArgumentError,"#{i+1}-th GPhys do not have dim '#{nm}'"
+	      end
+	      if g.coord(nm).length != sh[i]
+		raise ArgumentError,"loop dimensions must have the same lengths(#{nm}; #{sh[i]} vs #{g.coord(nm).length})"
+	      end
+	    end
+	  end
+	end
+
 	cs = [1]
 	(1...sh.length).each{|i| cs[i] = sh[i-1]*cs[i-1]}
 	idx_hash = Hash.new
@@ -96,20 +101,21 @@ module NumRu
 		  raise "Dimension '#{nm}' has been eliminated. "+
                         "You must keep all loop dimensions." 
 		end
-		grid.set_axis(nm,gphyses[j].axis(nm))
+		grid.set_axis(nm,gphyses[0].axis(nm))
 	      }
 	      io_module.write_grid(fl, grid)
 	      fl = files.shift if files.length >= 1
 	      results_whole.push( 
-		io_module.def_var(fl, rs.name, rs.data.ntype, 
-				  grid.axnames, rs.data)
+		GPhys.new( grid,
+			   io_module.def_var(fl, rs.name, rs.data.ntype, 
+				             grid.axnames, rs.data)
+			  )
 	      )
 	    end
 	  end
-	  idx = grid.axnames.collect{|k| idx_hash[k] || true}
 	  for j in 0...results.length
 	    rs = results[j]
-	    results_whole[j][*idx] = rs.data
+	    results_whole[j][idx_hash] = rs.data
 	  end
 	end
 	return results_whole
