@@ -11,6 +11,15 @@ require "numru/gphys/gphys_io_common"
 a NetCDF read/write helper by automatically interpreting conventions
 
 ==Module functions
+---is_a_NetCDF?(filename)
+    test whether the file is a NetCDF file.
+
+    ARGUMENTS
+    * filename (String): filename to test.
+
+    RETURN VALUE
+    * true/false
+
 ---open(files, varname)
     a GPhys constructor from a NetCDF file (or multiple NetCDF files).
 
@@ -110,7 +119,11 @@ a NetCDF read/write helper by automatically interpreting conventions
     (which is the return value of the block) are written in ((|files|)).
 
     ARGUMENTS
-    * gphyses (GPhys or Array of GPhys): GPhys object(s) to be processed
+    * gphyses (GPhys or Array of GPhys): GPhys object(s) to be processed.
+      All of them must have dimensions spcified with ((|loopdims|)),
+      and their lengths must not vary among files. Other dimensions
+      are aribtary, so, for example,  ((|gphyses|)) could be 
+      [a(lon,lat,time), b(lat,time)] as long as loopdims==["time"].
     * files (NetCDF or Array of NetCDF): the file in which the results are
       written. The number of the file must be smaller than or equalt to 
       the number of resultant GPhys objects (following the multiple assignment
@@ -128,7 +141,8 @@ a NetCDF read/write helper by automatically interpreting conventions
 
     The following raise exceptions (in adition to errors in arguments).
 
-    * Shape of the GPhys objects in ((|gphyses|)) are not the same.
+    * Dimensions specified by ((|loopdims|)) are not shared among
+      GPhys objects in ((|gphyses|)).
     * Return value of the block is not an Array of GPhys.
     * Dimension(s) used for looping (((|loopdims|))) is(are) eliminated
       from the retunred GPhys objects.
@@ -258,6 +272,24 @@ a NetCDF read/write helper by automatically interpreting conventions
     RETURN VALUE
     * convention (Module)
 
+---var_names(file)
+    ARGUMENTS
+    * file (NetCDF or String): if string,
+      it must be the name (path) of a NetCDF file.
+
+    RETURN VALUE
+    * names of variables (Array): this return the names of variables
+      which the file has.
+
+---var_names_except_coordinate(file)
+    ARGUMENTS
+    * file (NetCDF or String): if string,
+      it must be the name (path) of a NetCDF file.
+
+    RETURN VALUE
+    * names of variables (Array): this return the names of variables
+      which the file has, except variables for coordinate.
+
 =end
 
 module NumRu
@@ -268,6 +300,17 @@ module NumRu
 
       module_function
 
+      def is_a_NetCDF?(filename)
+        file = nil
+        begin
+          file = File.open(filename)
+          str = file.read(3)
+        ensure
+          file.close if file
+        end
+        return str=="CDF"
+      end
+
       def open(files, varname)
 	files, ncvar0 = __interpret_files( files, varname )
 	data = __files2varray( files, varname )
@@ -376,6 +419,61 @@ module NumRu
 					 &block)
       end
 
+      def var_names(files)
+	case files
+	when NetCDF
+          file = files
+	  opened = true
+	when String
+	  file = NetCDF.open(files)
+	  opened = false
+	when NArray, Array, Regexp
+	  files = NArray[ *files ] if files.is_a?(Array)
+	  files = GPhys::NetCDF_IO.__to_na_of_netcdf( files )
+	  files = GPhys::NetCDF_IO.__files_dim_matching( files, varname )
+	  file = files[0]
+	  files.length.times{|i|
+	    files[i].close unless i==0
+          }
+	  opened = false
+	else
+	  raise TypeError, "argument files: not a NetCDF, String, NArray, Array, or Regexp"
+	end
+	var_names = file.var_names
+	file.close unless opened
+	return var_names
+      end
+      def var_names_except_coordinates(files)
+	case files
+	when NetCDF
+          file = files
+	  opened = true
+	when String
+	  file = NetCDF.open(files)
+	  opened = false
+	when NArray, Array, Regexp
+	  files = NArray[ *files ] if files.is_a?(Array)
+	  files = GPhys::NetCDF_IO.__to_na_of_netcdf( files )
+	  files = GPhys::NetCDF_IO.__files_dim_matching( files, varname )
+	  file = files[0]
+	  files.length.times{|i|
+	    files[i].close unless i==0
+          }
+	  opened = false
+	else
+	  raise TypeError, "argument files: not a NetCDF, String, NArray, Array, or Regexp"
+	end
+        var_names = Array.new
+        file.var_names.each{|name|
+          f,var = __interpret_files( files, name )
+          if var.rank>1 || var.name!=var.dim_names[0]
+            var_names.push(name)
+          end
+        }
+	file.close unless opened
+        return var_names
+      end
+
       ############################################################
 
       def __files2varray( files, varname, dim=nil )
@@ -613,14 +711,21 @@ if $0 == __FILE__
     [sub.mean(0)]
   }
   f.close
-  f=NetCDF.create('tmpE2.nc')
-  GPhys::NetCDF_IO.each_along_dims_write([temp,temp_edy], f, "level"){|s1,s2|
-    [s1.mean(0),s2.mean(1).rename('T_edy')]
-  }
-  f.close
   f=NetCDF.create('tmpE0.nc')
   GPhys::NetCDF_IO.write( f, temp.mean(0) )
   f.close
 
   print `ncdump tmpE0.nc > tmpE0; ncdump tmpE1.nc > tmpE1 ; diff -u tmpE[01]`
+
+  f=NetCDF.create('tmpE2.nc')
+  GPhys::NetCDF_IO.each_along_dims_write([temp,temp_edy], f, "level"){|s1,s2|
+    [s1.mean(0),s2.mean(1).rename('T_edy')]
+  }
+  f.close
+
+  f=NetCDF.create('tmpE3.nc')
+  GPhys::NetCDF_IO.each_along_dims_write([temp,temp_xmean], f, "level"){|s1,s2|
+    [s1.mean(1),s2.rename('T_x_mean'),s2.mean(0).rename('T_xy_mean')]
+  }
+  f.close
 end
