@@ -294,7 +294,8 @@ It is intended to behave as a correspondent of a NetCDFVar object.
 =end
 
 require "date"
-require "narray"
+#require "narray"
+require "narray_miss"
 
 class GrADS_Gridded
 
@@ -302,13 +303,14 @@ class GrADS_Gridded
     alias open new
 
     def create(ctlfilename,noclobber=false,share=false)
-      if(noclobber)
-        raise "noclobber = true is not supported."
-      end
+      #if(noclobber)
+      #  raise "noclobber = true is not supported."
+      #end
       if(share)
         raise "share = true is not supported."
       end
-      if (File.exists?(ctlfilename))
+      #if (File.exists?(ctlfilename))
+      if(!noclobber && File.exists?(ctlfilename))
         print "#{ctlfilename} already exists.\n"
         print "overwrite #{ctlfilename} (y/n)? "
         ans = gets[0].chr
@@ -515,8 +517,7 @@ EOS
   end
 
   def put(ary)
-#    @options["big_endian"] = true
-    ary = ary.hton if ( @options["big_endian"] || @options["cray_32bit_ieee"] )
+    ary = ary.to_na.hton if ( @options["big_endian"] )
     if File.exist?(@dset)
       print "overwrite "+@dset+" (y/n)? "
       ans = gets[0].chr
@@ -532,7 +533,12 @@ EOS
 
   def get(name, level, time)
     if ( @options["template"] )
-      time_a = get_dim(@dimensions[3]) # julian day(integer) + time of day
+#      time_a = get_dim(@dimensions[3]) # julian day(integer) + time of day
+      time_a = get_dim(@dimensions[3])
+#      date0 = Date.parse(@dimensions[3][:units]).jd # date of the initial record
+      date0 = @dimensions[3][:startjd] # (julian day+time of day) of the initial record
+      time_a = time_a + date0
+
 #      time_d = Date.new1(time_a[time]-0.5) # precise julian day for 00UTC
       time_d = Date.new1(time_a[time].floor) # julian day(integer)
       init_year = time_d.year
@@ -570,7 +576,9 @@ EOS
     end
     @datafile.pos=start_byte
 
-    ary = NArray.to_na(@datafile.read(@xybytes), "sfloat",
+#    ary = NArray.to_na(@datafile.read(@xybytes), "sfloat",
+#                       @dimensions[0][:len],@dimensions[1][:len])
+    ary = NArray.to_na(@datafile.read(@xy*@map[name][:byte]), @map[name][:type],
                        @dimensions[0][:len],@dimensions[1][:len])
     ary = ary.ntoh if ( @options["big_endian"] || @options["cray_32bit_ieee"] )
     ary = ary.vtoh if ( @options["little_endian"] )
@@ -610,14 +618,20 @@ EOS
         when 'day'
           var_dim = var_dim.indgen!*dim[:increment]+dim[:start]
         when 'month'
+#          date0 = Date.parse(dim[:units]).jd # date of the initial record
+          date0 = dim[:startjd] # (julian day+time of day) of the initial record
           var_dim[0] = dim[:start]
           for i in 1..dim[:len]-1
-            var_dim[i] = ( Date.new1(var_dim[i-1]) >> dim[:increment] ).jd
+#            var_dim[i] = ( Date.new1(var_dim[i-1]) >> dim[:increment] ).jd
+            var_dim[i] = ( Date.jd(var_dim[i-1]+date0) >> dim[:increment] ).jd - date0
           end
         when 'year'
+#          date0 = Date.parse(dim[:units]).jd # date of the initial record
+          date0 = dim[:startjd] # (julian day+time of day) of the initial record
           var_dim[0] = dim[:start]
           for i in 1..dim[:len]-1
-            var_dim[i] = ( Date.new1(var_dim[i-1]) >> dim[:increment]*12 ).jd
+#            var_dim[i] = ( Date.new1(var_dim[i-1]) >> dim[:increment]*12 ).jd
+            var_dim[i] = ( Date.jd(var_dim[i-1]+date0) >> dim[:increment]*12 ).jd - date0
           end
         else
           raise "invalid units: "+dim[:increment_units]
@@ -758,12 +772,17 @@ EOS
             # idim = 3 --- time
             if (dim[:flag] =~ /LINEAR/i)
               start,increment= spec.split
-              julian_day = parse_starttime(start)
-              dim[:start] = julian_day
-              dim[:start_units] = 'days since '+Date.new1(0.0).to_s+' 00UTC'
+#              julian_day = parse_starttime(start)
+#              dim[:start] = julian_day
+#              dim[:start_units] = 'days since '+Date.new1(0.0).to_s+' 00UTC'
+              year,mon,day,hour,min = parse_starttime(start)
+              dim[:start] = 0.0
+#              dim[:start_units] = "days since #{year}-#{mon}-#{day} #{hour}:#{min}:0 +0:00"
+              dim[:startjd] = Date.new(year,mon,day).jd + hour/24.0 + min/1440.0
               dim[:increment],dim[:increment_units] = parse_timeincrement(increment)
 #              dim[:units] = dim[:increment_units]
-              dim[:units] = dim[:start_units]
+#              dim[:units] = dim[:start_units]
+              dim[:units] = "days since #{year}-#{mon}-#{day} #{hour}:#{min}:0 +0:00"
             else
               raise "invalid dimension flag(only LINEAR is available for time)"
             end
@@ -773,7 +792,8 @@ EOS
           raise "Invalid line: "+line
         end
       when /^\s*VARS/i
-        total_levs=0
+#        total_levs=0
+        total_lev_bytes=0
         while ( vline = @ctlfile.gets )
           case(vline)
           when /^\s*\*/,/^\s*$/
@@ -781,40 +801,60 @@ EOS
           when /^\s*ENDVARS/i
             break
           else
-#            vline =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+.*?)\s*$/
-            vline =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(\S+.*?)\s*$/
+            vline =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+.*?)\s*$/
+#            vline =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(\S+.*?)\s*$/
             if( !($1 && $2 && $3 && $4) )
               raise "Something is wrong with this line: "+vline
             end
+            name = $1; option = $3; description = $4; units = $5
             nlev = max($2.to_i,1)
-            total_levs += nlev
+#            total_levs += nlev
+#            case ($3)
+            case (option)
+            when /^0/,/^99/,/^-1,30/
+              byte=4; type="sfloat"
+            when /^-1,40,4/
+              byte=4; type="int"
+            when /^-1,40,2,-1/
+              byte=2; type="sint"
+            when /^-1,40,1/
+              byte=1; type="byte"
+            else
+              p option
+              raise "invalid or unsupported variable placement option: "
+            end
+            total_lev_bytes += nlev*byte
 #            @variables.push({:name=>$1,:nlev=>nlev,:option=>$3,
 #                              :description=>$4})
             if( ! $5 ) 
-              @variables.push({:name=>$1,:nlev=>nlev,:option=>$3,
-                                :description=>$4})
+              @variables.push({:name=>name,:nlev=>nlev,:option=>$3,:type=>type,:byte=>byte,
+                                :description=>description})
             else
-              @variables.push({:name=>$1,:nlev=>nlev,:option=>$3,
-                                :description=>$4, :units=>$5})
+              @variables.push({:name=>$1,:nlev=>nlev,:option=>$3,:type=>type,:byte=>byte,
+                                :description=>description, :units=>$5})
             end
           end
         end
         @map=Hash.new
-        cum_lev = 0
+#        cum_lev = 0
+        cum_lev_bytes = 0
         @variables.each{|i|
           varname = i[:name]
-          case (i[:option])
-          when /^0/,/^99/,/^-1,30/
+#          case (i[:option])
+#          when /^0/,/^99/,/^-1,30/
             @map[varname]={:offset=>@fileheader_len,:nlev=>i[:nlev],
-              :start=>cum_lev,:zstep=>1,:tstep=>total_levs}
+#              :start=>cum_lev,:zstep=>1,:tstep=>total_levs}
+              :start=>cum_lev_bytes,:zstep=>i[:byte],:tstep=>total_lev_bytes, 
+              :type=>i[:type],:byte=>i[:byte]}
             if i[:option] =~ /^-1,30/
               @map[varname][:xytranspose]=true
             end
-            #p "## test - #{varname}",@map[varname]
-          else
-            raise "invalid or unsupported variable placement option: "+i[:option]
-          end
-          cum_lev += i[:nlev]
+#            p "## test - #{varname}",@map[varname]
+#          else
+#            raise "invalid or unsupported variable placement option: "+i[:option]
+#          end
+#          cum_lev += i[:nlev]
+          cum_lev_bytes += i[:nlev]*i[:byte]
         }
       end
 
@@ -830,6 +870,7 @@ EOS
 
     #<post processing>
     @xybytes = 4 * @dimensions[0][:len] * @dimensions[1][:len]
+    @xy = @dimensions[0][:len] * @dimensions[1][:len]
 
   end
 
@@ -843,8 +884,10 @@ EOS
       if (time<0 || time>=@dimensions[3][:len])
         raise "Time #{time} is not in the data period"
       end
+#      iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
+#      str_byte = map[:offset] + @xybytes*iblock
       iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
-      str_byte = map[:offset] + @xybytes*iblock
+      str_byte = map[:offset] + @xy*iblock
       if( @options["sequential"] )
         str_byte += iblock*8 + 4
       end
@@ -906,12 +949,9 @@ EOS
       raise "The date part must be [dd]mmmyyyy, but what was given is: "+sdate
     end
 
-    #print '## full start time: ',shour+':'+smin+'Z'+sday+smon+syear,"\n"
-    #print '## ',date.to_s,' ',timeofday,"\n"
-
-    julian_day = date.jd + timeofday
-
-    return julian_day
+#    julian_day = date.jd + timeofday
+#    return julian_day
+    return year,mon,day,hour,min
   end
 
   def parse_timeincrement(string)
@@ -1040,12 +1080,30 @@ class GrADSVar
   end
 
   def vartype
-    "sfloat"
+    if( @idim == -1 )
+      for i in 0...@ctl.variables.length
+        if @varname == @ctl.variables[i][:name]
+          type = @ctl.variables[i][:type]
+        end
+      end
+    else
+      type = "sfloat"
+    end
+    type
   end
   alias ntype vartype
 
   def typecode
-    4
+    if( @idim == -1 )
+      for i in 0...@ctl.variables.length
+        if @varname == @ctl.variables[i][:name]
+          byte = @ctl.variables[i][:byte]
+        end
+      end
+    else
+      byte = 4
+    end
+    byte
   end
 
   def att(name)
@@ -1168,9 +1226,16 @@ class GrADSVar
 #          na_xy = @ctl.get(@varname,lev,time).swap_byte
           na_xy = @ctl.get(@varname,lev,time)
           if time == str_t && lev == str_z
-            na = NArray.sfloat(end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
+#            na = NArray.sfloat(end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
+#            na = NArrayMiss.sfloat(end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
+            na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
           end
-          na[true,true,lev-str_z,time-str_t] = na_xy[str_x..end_x,str_y..end_y]
+#          na[true,true,lev-str_z,time-str_t] = na_xy[str_x..end_x,str_y..end_y]
+#          mask = ( na_xy[str_x..end_x,str_y..end_y].abs < @ctl.undef.abs/2 )
+          mask = ( na_xy[str_x..end_x,str_y..end_y].ne @ctl.undef )
+#          mask = ( na_xy[str_x..end_x,str_y..end_y] > -100.0 )
+          na[true,true,lev-str_z,time-str_t] = NArrayMiss.to_nam(na_xy[str_x..end_x,str_y..end_y],mask)
+#          na[true,true,lev-str_z,time-str_t].set_mask(mask)
         end
       end
     end
